const TRACK_ATTRIBUTE = 'data-track'
const PROPERTIES_ATTRIBUTE = 'data-track-properties'
const OPTIONS_ATTRIBUTE = 'data-track-options'

const eventMap = []

const attributeToJson = ({attributeKey, el}) => {
  const data = el.getAttribute(attributeKey)
  return data ? JSON.parse(data.replace(/&quot;/g, '"')) : {}
}

const trackingCallbackFactory = el =>
  function() {
    window.rubik.analytics.track(
      el.getAttribute(TRACK_ATTRIBUTE),
      attributeToJson({attributeKey: PROPERTIES_ATTRIBUTE, el}),
      attributeToJson({attributeKey: OPTIONS_ATTRIBUTE, el})
    )
  }

const listenToClicksToTrack = function(eventFn) {
  const elements = document.querySelectorAll(`[${TRACK_ATTRIBUTE}]`)

  Array.from(elements).forEach((element, idx) => {
    const callback = trackingCallbackFactory(element)
    eventMap.push({element, callback})
    element.addEventListener('click', callback)
  })
}

const removeListeners = function() {
  eventMap.forEach(({element, callback}) => {
    element.removeEventListener('click', callback)
  })
}

document.addEventListener('segment:reload', function() {
  removeListeners()
  listenToClicksToTrack()
})

export default listenToClicksToTrack
