import axios from 'axios'
import {ADOBE_ORG_ID} from './config'

const DEMDEX_URL = 'https://mpispain.demdex.net/event'
const LOCAL_STORAGE_KEY = 'dmp:track'

const paramsFactory = ({visitorId}) => ({
  d_mid: visitorId,
  d_orgid: ADOBE_ORG_ID,
  d_rtbd: 'json',
  d_full: 1,
  d_cts: 2
})

const shouldSync = dmpResponse =>
  dmpResponse.stuff && dmpResponse.stuff[0] && Boolean(dmpResponse.stuff[0].cv)

const getSyncIds = dmpResponse =>
  dmpResponse.stuff[0].cv
    .split(/(\d+)/)
    .map(str => +str)
    .filter(Boolean)

const defaultImageFactory = () => new window.Image()
const defaultStorageFactory = () => window.localStorage

const googleSync = ({ids, imageFactory = defaultImageFactory}) => {
  const googleAdsImg = imageFactory()
  googleAdsImg.src = `https://googleads.g.doubleclick.net/pagead/viewthroughconversion/1024323628/?guid=ON&script=0&data=aam=${ids.join(
    ';aam='
  )}&value=0`
  googleAdsImg.onload = function() {
    console.log('segment in google Ads created', ids)
  }
  googleAdsImg.onerror = function() {
    console.log('error creating segments in google ads')
  }
}

const facebookSync = ({ids, imageFactory = defaultImageFactory}) => {
  const facebookAdsImg = imageFactory()
  facebookAdsImg.src = `https://www.facebook.com/tr/?id=283351338713870&ev=Adobe-Audience-Manager-Segment&cd[segId]=${ids.join(
    ','
  )}&noscript=1`
  facebookAdsImg.onload = function() {
    console.log('segment in fb Ads created', ids)
  }
  facebookAdsImg.onerror = function() {
    console.log('error creating segments in fb ads')
  }
}

const track = ({visitorId}) =>
  axios.get(DEMDEX_URL, {
    params: paramsFactory({visitorId})
  })

const persist = ({visitorId, storageFactory = defaultStorageFactory}) =>
  storageFactory().setItem(LOCAL_STORAGE_KEY, visitorId)

const clean = ({storageFactory = defaultStorageFactory}) =>
  storageFactory().setItem(LOCAL_STORAGE_KEY, '')

const sync = async ({visitorId, imageFactory, storageFactory}) => {
  try {
    persist({visitorId, storageFactory})
    const dmpResponse = await track({visitorId})
    if (shouldSync(dmpResponse)) {
      const ids = getSyncIds(dmpResponse)
      googleSync({ids, imageFactory})
      facebookSync({ids, imageFactory})
    }
    clean({storageFactory})
  } catch (error) {
    console.error('rubik:dmp-track', error)
  }
}

export const onLoad = ({
  imageFactory,
  storageFactory = defaultStorageFactory
} = {}) => {
  const visitorId = storageFactory().getItem(LOCAL_STORAGE_KEY)
  if (!visitorId) return
  sync({visitorId, imageFactory, storageFactory})
}

export default sync
