import sinon from 'sinon'
import listenToClicksToTrack from '../../src/analytics/clickListener'

describe('#clickListener', () => {
  const FAKE_EVENT = 'myFakeEvent'
  const FAKE_ID = 'myFakeId'
  const FAKE_PROPS = '{&quot;section&quot;:&quot;candidate&quot;}'
  const FAKE_OPTIONS = '{&quot;protocols&quot;:{&quot;event_version&quot;:2}}'
  let trackStub
  let buttonEl

  before(() => {
    window.rubik = window.rubik || {}
    window.rubik.analytics = window.rubik.analytics || {track: function() {}}
  })

  describe('when event, properties and options are set', () => {
    before(() => {
      trackStub = sinon.stub(window.rubik.analytics, 'track')
      const fixture = `
    <button
      id="${FAKE_ID}"
      data-track="${FAKE_EVENT}"
      data-track-options="${FAKE_OPTIONS}"
      data-track-properties="${FAKE_PROPS}"
      >
        Click me!
    </button>
    `
      document.body.insertAdjacentHTML('afterbegin', fixture)
      listenToClicksToTrack()
      buttonEl = document.getElementById(FAKE_ID)
    })

    after(() => {
      buttonEl.parentNode.removeChild(buttonEl)
      trackStub.restore()
    })

    it('should track the right event with its props and its options', () => {
      buttonEl.click()
      sinon.assert.calledWithExactly(
        trackStub,
        FAKE_EVENT,
        {
          section: 'candidate'
        },
        {
          protocols: {event_version: 2}
        }
      )
    })
  })
  describe('when event and properties are set', () => {
    before(() => {
      trackStub = sinon.stub(window.rubik.analytics, 'track')
      const fixture = `
    <button
      id="${FAKE_ID}"
      data-track="${FAKE_EVENT}"
      data-track-properties="${FAKE_PROPS}"
      >
        Click me!
    </button>
    `
      document.body.insertAdjacentHTML('afterbegin', fixture)
      listenToClicksToTrack()
      buttonEl = document.getElementById(FAKE_ID)
    })

    after(() => {
      buttonEl.parentNode.removeChild(buttonEl)
      trackStub.restore()
    })

    it('should track the right event with its props', () => {
      buttonEl.click()
      sinon.assert.calledWithExactly(
        trackStub,
        FAKE_EVENT,
        {
          section: 'candidate'
        },
        {}
      )
    })
  })
  describe('when event is set without properties', () => {
    before(() => {
      trackStub = sinon.stub(window.rubik.analytics, 'track')
      const fixture = `
    <button
      id="${FAKE_ID}"
      data-track="${FAKE_EVENT}"
      >
        Click me!
    </button>
    `
      document.body.insertAdjacentHTML('afterbegin', fixture)
      listenToClicksToTrack()
      buttonEl = document.getElementById(FAKE_ID)
    })

    after(() => {
      buttonEl.parentNode.removeChild(buttonEl)
      trackStub.restore()
    })

    it('should track the right event with no props', () => {
      buttonEl.click()
      sinon.assert.calledWithExactly(trackStub, FAKE_EVENT, {}, {})
    })
  })

  describe('when no event is set ', () => {
    before(() => {
      trackStub = sinon.stub(window.rubik.analytics, 'track')
      const fixture = `
    <button
      id="${FAKE_ID}"
      >
        Click me!
    </button>
    `
      document.body.insertAdjacentHTML('afterbegin', fixture)
      listenToClicksToTrack()
      buttonEl = document.getElementById(FAKE_ID)
    })

    after(() => {
      buttonEl.parentNode.removeChild(buttonEl)
      trackStub.restore()
    })

    it('should not call the track function', () => {
      buttonEl.click()
      sinon.assert.callCount(trackStub, 0)
    })
  })
})
